/**
 * Saeki(Nekketu Saiten Kyousi Saeki)
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * gamemanager.c(manage game status)
 */

GameStatus gameStatus;

long gameStartSecond;
int level;
int leftAnsNmb;

// number of status messages
#define MSG_NMB 8

char okMsg[MSG_NMB][11] = {
  "",
  "̒q",
  "",
  "",
  "ׂ肮",
  "悵",
  "",
  "iCX"};

char ngMsg[MSG_NMB][11] = { 
  "߂",
  "Ԃԁ[",
  "p",
  "Ƃ",
  "ȂƂ",
  "ւI",
  "͂I",
  "ił"};

char unMsg[MSG_NMB][11] = { 
  "??x",
  "??킩",
  "??Ȃɂ",
  "??ۂȂ́H",
  "??~Ȃ́H",
  "??ǂH",
  "??قH",
  "??H"};

char titleMsg[30] = {
  "M̓_t"};

// rank messages
char tlvStr[8][9] = {
  "NY", "_", 
  "{P", "l", 
  "Ă", 
  "JX}", "", "^M"}; 

// gameover message
char goMsg[20] = {
  "͂܂"};

static void initGame() {
  gameStatus.waitTick = 0;
  gameStatus.status = TITLE;
  FrmGotoForm(TITLE_FORM);
}

static void startButtonPushed() {
  gameStartSecond = TimGetSeconds();
  gameStatus.status = GAME_IN_PROGRESS;
  FrmGotoForm(SAEKI_FORM);
}

static void startGame() {
  gameStatus.drawPaper = TRUE;
  level = 0;
}

/**
 * set wait interval(ticks)
 */
static void setWaitTick(long interval) {
  gameStatus.waitTick = TimGetTicks() + interval;
}

static int abs(int n) {
  if ( n<0 ) return -n;
  else return n;
}

static void drawTitle() {
  WinDrawChars(titleMsg, StrLen(titleMsg), 
	       80-FntCharsWidth(titleMsg, StrLen(titleMsg))/2, 75);
}

/**
 * make result rank message
 */
static void setResult(int lv) {
  char nmbStr[4];
  int rnk = 7;
  StrCopy(titleMsg, "x ");
  StrIToA(nmbStr, lv);
  StrCat(titleMsg, nmbStr);
  StrCat(titleMsg, " ");
  if ( lv < 10 ) rnk = 0;
  else if ( lv < 20 ) rnk = 1;
  else if ( lv < 30 ) rnk = 2;
  else if ( lv < 45 ) rnk = 3;
  else if ( lv < 60 ) rnk = 4;
  else if ( lv < 80 ) rnk = 5;
  else if ( lv < 100 ) rnk = 6;
  StrCat(titleMsg, tlvStr[rnk]);
  StrCat(titleMsg, "t");
}

/**
 * draw status message
 */
static void drawMessage(char *msg) {
  RectangleType rct;
  rct.topLeft.x = 0;
  rct.topLeft.y = 150;
  rct.extent.x = 160;
  rct.extent.y = 10;
  WinEraseRectangle(&rct, 0);
  WinDrawChars(msg, StrLen(msg), 0, 150);
}

void drawQuestion(int x, int y) {
  int cy = y-10;
  if ( cy < PEN_TOP_Y ) cy = PEN_TOP_Y;
  if ( cy > PEN_BOTTOM_Y-12 ) cy = PEN_BOTTOM_Y-12;
  WinDrawChars("?", 1, x, cy);
}

void drawOk(int x, int y) {
  int cy = y-10;
  if ( cy < PEN_TOP_Y ) cy = PEN_TOP_Y;
  if ( cy > PEN_BOTTOM_Y-12 ) cy = PEN_BOTTOM_Y-12;
  WinDrawChars("OK", 2, x, cy);
}

void drawNg(int x, int y) {
  int cy = y-10;
  if ( cy < PEN_TOP_Y ) cy = PEN_TOP_Y;
  if ( cy > PEN_BOTTOM_Y-12 ) cy = PEN_BOTTOM_Y-12;
  WinDrawChars("NG", 2, x, cy);
}

void eraseAnswer(int idx) {
  int x = 0;
  if ( LEFT_HAND ) x = 140;
  WinDrawLine(x, blank[idx].y+4, x+18, blank[idx].y+4);
  WinDrawLine(x, blank[idx].y+7, x+18, blank[idx].y+7);
}

void gameover() {
    setResult(level);
    setWaitTick(300);
    gameStatus.status = GOTO_TITLE;
    gameStatus.drawPaper = FALSE;
}

// width and height of answer field
#define BLANK_WIDTH 30
#define BLANK_HEIGHT 10

/**
 * circle or check drawn
 */
static void locusDrawn(int lcStat) {
  int i;
  int distMin = 9999, dist;
  int blIdx = -1;
  
  if ( lcStat == 2 ) {
    drawQuestion(lcXmin, lcYmax);
    drawMessage(unMsg[randN(MSG_NMB)]);
    return;
  }

  for ( i=0 ; i<blankNmb ; i++ ) {
    Blank bl = blank[i];
    if ( !(bl.x>lcXmax || bl.x+BLANK_WIDTH<lcXmin || 
	 bl.y>lcYmax || bl.y+BLANK_HEIGHT<lcYmin) ) {
      dist = abs((lcXmin+lcXmax)/2 - (bl.x+BLANK_WIDTH/2)) +
	abs((lcYmin+lcYmax)/2 - (bl.y+BLANK_HEIGHT/2));
      if ( dist < distMin ) {
	blIdx = i; distMin = dist;
      }
    }
  }
  if ( blIdx == -1 ) {
    drawQuestion(lcXmin, lcYmax);
    return;
  }

  if ( (blank[blIdx].corc^lcStat) ) {
    drawOk(lcXmin, lcYmax);
    eraseAnswer(blIdx);
    blank[blIdx].x = -9999;
    leftAnsNmb--;
    level++;
    if ( leftAnsNmb <= 0 ) {
      gameStatus.drawPaper = TRUE;
    }
    drawMessage(okMsg[randN(MSG_NMB)]);
  } else {
    drawNg(lcXmin, lcYmax);
    drawMessage(ngMsg[randN(MSG_NMB)]);
    gameover();
  }
}

// game limit time(seconds)
#define GAME_TIME_SECONDS 60

static void handleGameStatus() {
  if ( gameStatus.status == GAME_IN_PROGRESS &&
       TimGetSeconds() > (gameStartSecond+GAME_TIME_SECONDS) ) {
    drawMessage(goMsg);
    gameover();
  }
  if ( gameStatus.waitTick > 0 ) {
    if ( TimGetTicks() < gameStatus.waitTick ) return;
    gameStatus.waitTick = 0;
  }
  if ( gameStatus.drawPaper ) {
    drawPaper(level);
    leftAnsNmb = blankNmb;
    gameStatus.drawPaper = FALSE;
  }
  if ( gameStatus.status == GOTO_TITLE ) {
    clearPaper();
    initGame();
  }
}
